@@ -65,13 +65,13 @@ typedef struct {
    menu-replace
    tooltip-popup
    tooltip-popdown
-
-   TODO:
    drag-start
    drag-accept
    drag-fail
    expander-toggle-on
    expander-toggle-off
+
+   TODO:
    scroll-xxx
    window-switch
    window-resize-xxx
@@ -97,11 +97,15 @@ static guint
     signal_id_widget_window_state_event,
     signal_id_notebook_switch_page,
     signal_id_tree_view_cursor_changed,
-    signal_id_icon_view_selection_changed;
+    signal_id_icon_view_selection_changed,
+    signal_id_widget_drag_begin,
+    signal_id_widget_drag_failed,
+    signal_id_widget_drag_drop,
+    signal_id_expander_activate;
 
 static GQuark
     disable_sound_quark,
-    was_hidden_quark;
+    was_iconized_quark;
 
 /* Make sure GCC doesn't warn us about a missing prototype for this
  * exported function */
@@ -292,43 +296,64 @@ static SoundEventData* filter_sound_event(SoundEventData *d) {
     return d;
 }
 
-static gboolean is_hidden(GdkDisplay *d, GdkWindow *w) {
+static gint window_get_desktop(GdkDisplay *d, GdkWindow *w) {
     Atom type_return;
     gint format_return;
     gulong nitems_return;
     gulong bytes_after_return;
     guchar *data = NULL;
-    gboolean r = FALSE;
 
     if (XGetWindowProperty(GDK_DISPLAY_XDISPLAY(d), GDK_WINDOW_XID(w),
-                            gdk_x11_get_xatom_by_name_for_display(d, "_NET_WM_STATE"),
-                            0, G_MAXLONG, False, XA_ATOM, &type_return,
+                            gdk_x11_get_xatom_by_name_for_display(d, "_NET_WM_DESKTOP"),
+                            0, G_MAXLONG, False, XA_CARDINAL, &type_return,
                             &format_return, &nitems_return, &bytes_after_return,
-                            &data) != Success)
-        return FALSE;
+                           &data) != Success)
+        return -1;
 
-    if (type_return == XA_ATOM && format_return == 32 && data) {
-        unsigned i;
+    if (type_return == XA_CARDINAL && format_return == 32 && data) {
+        guint32 desktop = *(guint32*) data;
 
-        for (i = 0; i < nitems_return; i++) {
-            Atom atom = ((Atom*) data)[i];
-
-            if (atom == gdk_x11_get_xatom_by_name_for_display(d, "_NET_WM_STATE_HIDDEN")) {
-                r = TRUE;
-                break;
-            }
-        }
+        if (desktop != 0xFFFFFFFF)
+            return (gint) desktop;
     }
 
     if (type_return != None && data != NULL)
         XFree (data);
 
-    return r;
+    return -1;
+}
+
+static gint display_get_desktop(GdkDisplay *d) {
+    Atom type_return;
+    gint format_return;
+    gulong nitems_return;
+    gulong bytes_after_return;
+    guchar *data = NULL;
+
+    if (XGetWindowProperty(GDK_DISPLAY_XDISPLAY(d), DefaultRootWindow(GDK_DISPLAY_XDISPLAY(d)),
+                            gdk_x11_get_xatom_by_name_for_display(d, "_NET_CURRENT_DESKTOP"),
+                            0, G_MAXLONG, False, XA_CARDINAL, &type_return,
+                            &format_return, &nitems_return, &bytes_after_return,
+                            &data) != Success)
+        return -1;
+
+    if (type_return == XA_CARDINAL && format_return == 32 && data) {
+
+        guint32 desktop = *(guint32*) data;
+
+        if (desktop != 0xFFFFFFFF)
+            return (gint) desktop;
+    }
+
+    if (type_return != None && data != NULL)
+        XFree (data);
+
+    return -1;
 }
 
 static void dispatch_sound_event(SoundEventData *d) {
     int ret = CA_SUCCESS;
-    static gboolean menu_is_popped_up = TRUE;
+    static gboolean menu_is_popped_up = FALSE;
 
     if (g_object_get_qdata(d->object, disable_sound_quark))
         return;
@@ -343,7 +368,8 @@ static void dispatch_sound_event(SoundEventData *d) {
 
         if (is_menu_hint(hint)) {
 
-            if (menu_is_popped_up) {
+            if (!menu_is_popped_up) {
+
                 ret = ca_gtk_play_for_widget(GTK_WIDGET(d->object), 0,
                                              CA_PROP_EVENT_ID, "menu-popup",
                                              CA_PROP_EVENT_DESCRIPTION, "Menu popped up",
@@ -460,15 +486,26 @@ static void dispatch_sound_event(SoundEventData *d) {
 
     if (GTK_IS_WINDOW(d->object) && d->signal_id == signal_id_widget_window_state_event) {
         GdkEventWindowState *e;
-        gboolean h, ph;
+        gint w_desktop, c_desktop;
+        GdkDisplay *display;
 
         e = (GdkEventWindowState*) d->event;
 
-        h = is_hidden(gdk_screen_get_display(gdk_event_get_screen(d->event)), e->window);
-        ph = !!g_object_get_qdata(d->object, was_hidden_quark);
-        g_object_set_qdata(d->object, was_hidden_quark, GINT_TO_POINTER(h));
+        /* Unfortunately GDK_WINDOW_STATE_ICONIFIED is used both for
+         * proper minimizing and when a window becomes invisible
+         * because the desktop was switched. To handle this we check
+         * if the window becoming invisible is actually on the current
+         * desktop, and only if that's the case we assume it is being
+         * minimized. We then store this information, so that we know
+         * later on when the window is unminimized again. */
 
-        if ((e->changed_mask & GDK_WINDOW_STATE_ICONIFIED) && (e->new_window_state & GDK_WINDOW_STATE_ICONIFIED) && h && !ph) {
+        display = gdk_screen_get_display(gdk_event_get_screen(d->event));
+        w_desktop = window_get_desktop(display, e->window);
+        c_desktop = display_get_desktop(display);
+
+        if ((e->changed_mask & GDK_WINDOW_STATE_ICONIFIED) &&
+            (e->new_window_state & GDK_WINDOW_STATE_ICONIFIED) &&
+            w_desktop == c_desktop) {
 
             ret = ca_gtk_play_for_widget(GTK_WIDGET(d->object), 0,
                                          CA_PROP_EVENT_ID, "window-minimized",
@@ -476,7 +513,10 @@ static void dispatch_sound_event(SoundEventData *d) {
                                          CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                          NULL);
 
-        } else if ((e->changed_mask & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN)) && (e->new_window_state & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN))) {
+            g_object_set_qdata(d->object, was_iconized_quark, GINT_TO_POINTER(1));
+
+        } else if ((e->changed_mask & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN)) &&
+                   (e->new_window_state & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN))) {
 
             ret = ca_gtk_play_for_widget(GTK_WIDGET(d->object), 0,
                                          CA_PROP_EVENT_ID, "window-maximized",
@@ -484,14 +524,22 @@ static void dispatch_sound_event(SoundEventData *d) {
                                          CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                          NULL);
 
-        } else if ((e->changed_mask & GDK_WINDOW_STATE_ICONIFIED) && !(e->new_window_state & GDK_WINDOW_STATE_ICONIFIED) && ph) {
+            g_object_set_qdata(d->object, was_iconized_quark, GINT_TO_POINTER(0));
+
+        } else if ((e->changed_mask & GDK_WINDOW_STATE_ICONIFIED) &&
+                   !(e->new_window_state & GDK_WINDOW_STATE_ICONIFIED) &&
+                   g_object_get_qdata(d->object, was_iconized_quark)) {
 
             ret = ca_gtk_play_for_widget(GTK_WIDGET(d->object), 0,
                                          CA_PROP_EVENT_ID, "window-unminimized",
                                          CA_PROP_EVENT_DESCRIPTION, "Window unminimized",
                                          CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                          NULL);
-        } else if ((e->changed_mask & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN)) && !(e->new_window_state & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN))) {
+
+            g_object_set_qdata(d->object, was_iconized_quark, GINT_TO_POINTER(0));
+
+        } else if ((e->changed_mask & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN)) &&
+                   !(e->new_window_state & (GDK_WINDOW_STATE_MAXIMIZED|GDK_WINDOW_STATE_FULLSCREEN))) {
 
             ret = ca_gtk_play_for_widget(GTK_WIDGET(d->object), 0,
                                          CA_PROP_EVENT_ID, "window-unmaximized",
@@ -606,6 +654,7 @@ static void dispatch_sound_event(SoundEventData *d) {
                                     CA_PROP_EVENT_DESCRIPTION, "Tab changed",
                                     CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                     NULL);
+        goto finish;
     }
 
     if (GTK_IS_TREE_VIEW(d->object) && d->signal_id == signal_id_tree_view_cursor_changed) {
@@ -614,6 +663,7 @@ static void dispatch_sound_event(SoundEventData *d) {
                                     CA_PROP_EVENT_DESCRIPTION, "Item selected",
                                     CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                     NULL);
+        goto finish;
     }
 
     if (GTK_IS_ICON_VIEW(d->object) && d->signal_id == signal_id_icon_view_selection_changed) {
@@ -622,27 +672,82 @@ static void dispatch_sound_event(SoundEventData *d) {
                                     CA_PROP_EVENT_DESCRIPTION, "Item selected",
                                     CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                     NULL);
+        goto finish;
     }
 
-/*     if (ret != CA_SUCCESS) */
-/*         g_warning("Failed to play event sound: %s", ca_strerror(ret)); */
+    if (GTK_IS_EXPANDER(d->object) && d->signal_id == signal_id_expander_activate) {
+
+        if (gtk_expander_get_expanded(GTK_EXPANDER(d->object)))
+            ret = ca_gtk_play_for_event(d->event, 0,
+                                        CA_PROP_EVENT_ID, "expander-toggle-on",
+                                        CA_PROP_EVENT_DESCRIPTION, "Expander expanded",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                                        NULL);
+        else
+            ret = ca_gtk_play_for_event(d->event, 0,
+                                        CA_PROP_EVENT_ID, "expander-toggle-off",
+                                        CA_PROP_EVENT_DESCRIPTION, "Expander unexpanded",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                                        NULL);
+
+        goto finish;
+    }
+
+    if (GTK_IS_WIDGET(d->object)) {
+
+        if (d->signal_id == signal_id_widget_drag_begin) {
+
+            ret = ca_gtk_play_for_event(d->event, 0,
+                                        CA_PROP_EVENT_ID, "drag-start",
+                                        CA_PROP_EVENT_DESCRIPTION, "Drag started",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                                        NULL);
+            goto finish;
+
+        } else if (d->signal_id == signal_id_widget_drag_drop) {
+
+            ret = ca_gtk_play_for_event(d->event, 0,
+                                        CA_PROP_EVENT_ID, "drag-accept",
+                                        CA_PROP_EVENT_DESCRIPTION, "Drag accepted",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                                        NULL);
+            goto finish;
+
+        } else if (d->signal_id == signal_id_widget_drag_failed) {
+
+            ret = ca_gtk_play_for_event(d->event, 0,
+                                        CA_PROP_EVENT_ID, "drag-fail",
+                                        CA_PROP_EVENT_DESCRIPTION, "Drag failed",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                                        NULL);
+            goto finish;
+        }
+    }
+
+finish:
+
+    ;
+    /* if (ret != CA_SUCCESS) */
+    /*     g_warning("Failed to play event sound: %s", ca_strerror(ret)); */
 }
 
-static gboolean idle_cb(void *userdata) {
+static void dispatch_queue(void) {
     SoundEventData *d;
 
-    idle_id = 0;
-
     while ((d = g_queue_pop_head(&sound_event_queue))) {
 
         if (!(d = filter_sound_event(d)))
             continue;
 
-/*         g_message("Dispatching signal %s on %s", g_signal_name(d->signal_id), g_type_name(G_OBJECT_TYPE(d->object))); */
-
         dispatch_sound_event(d);
         free_sound_event(d);
     }
+}
+
+static gboolean idle_cb(void *userdata) {
+    idle_id = 0;
+
+    dispatch_queue();
 
     return FALSE;
 }
@@ -661,6 +766,17 @@ static gboolean emission_hook_cb(GSignalInvocationHint *hint, guint n_param_valu
 
     object = g_value_get_object(&param_values[0]);
 
+    /* g_message("signal '%s' on object of type '%s' with name '%s'", */
+    /*           g_signal_name(hint->signal_id), */
+    /*           G_OBJECT_TYPE_NAME(object), */
+    /*           gtk_widget_get_name(GTK_WIDGET(object))); */
+
+    /* if (GTK_IS_WINDOW(object)) */
+    /*     g_message("window role='%s' title='%s' type='%u'", */
+    /*               gtk_window_get_role(GTK_WINDOW(object)), */
+    /*               gtk_window_get_title(GTK_WINDOW(object)), */
+    /*               gtk_window_get_type_hint(GTK_WINDOW(object))); */
+
     /* Filter a few very often occuring signals as quickly as possible */
     if ((hint->signal_id == signal_id_widget_hide ||
          hint->signal_id == signal_id_widget_show ||
@@ -673,8 +789,6 @@ static gboolean emission_hook_cb(GSignalInvocationHint *hint, guint n_param_valu
         !GTK_WIDGET_DRAWABLE(object))
         return TRUE;
 
-/*     g_message("signal %s on %s", g_signal_name(hint->signal_id), g_type_name(G_OBJECT_TYPE(object))); */
-
     d = g_slice_new0(SoundEventData);
 
     d->object = g_object_ref(object);
@@ -714,9 +828,12 @@ static void install_hook(GType type, const char *sig, guint *sn) {
 static void read_enable_input_feedback_sounds(GtkSettings *s) {
     gboolean enabled = !disabled;
 
-    g_object_get(G_OBJECT(s), "gtk-enable-input-feedback-sounds", &enabled, NULL);
-
-    disabled = !enabled;
+    if (g_getenv("CANBERRA_FORCE_INPUT_FEEDBACK_SOUNDS"))
+        disabled = FALSE;
+    else {
+        g_object_get(G_OBJECT(s), "gtk-enable-input-feedback-sounds", &enabled, NULL);
+        disabled = !enabled;
+    }
 }
 
 static void enable_input_feedback_sounds_changed(GtkSettings *s, GParamSpec *arg1, gpointer userdata) {
@@ -742,11 +859,16 @@ static void connect_settings(void) {
     connected = TRUE;
 }
 
+static gboolean quit_handler(gpointer data) {
+    dispatch_queue();
+    return FALSE;
+}
+
 G_MODULE_EXPORT void gtk_module_init(gint *argc, gchar ***argv[]) {
 
     /* This is the same quark libgnomeui uses! */
     disable_sound_quark = g_quark_from_string("gnome_disable_sound_events");
-    was_hidden_quark = g_quark_from_string("canberra_was_hidden");
+    was_iconized_quark = g_quark_from_string("canberra_was_iconized");
 
     /* Hook up the gtk setting */
     connect_settings();
@@ -763,6 +885,12 @@ G_MODULE_EXPORT void gtk_module_init(gint *argc, gchar ***argv[]) {
     install_hook(GTK_TYPE_NOTEBOOK, "switch-page", &signal_id_notebook_switch_page);
     install_hook(GTK_TYPE_TREE_VIEW, "cursor-changed", &signal_id_tree_view_cursor_changed);
     install_hook(GTK_TYPE_ICON_VIEW, "selection-changed", &signal_id_icon_view_selection_changed);
+    install_hook(GTK_TYPE_WIDGET, "drag-begin", &signal_id_widget_drag_begin);
+    install_hook(GTK_TYPE_WIDGET, "drag-drop", &signal_id_widget_drag_drop);
+    install_hook(GTK_TYPE_WIDGET, "drag-failed", &signal_id_widget_drag_failed);
+    install_hook(GTK_TYPE_EXPANDER, "activate", &signal_id_expander_activate);
+
+    gtk_quit_add(1, quit_handler, NULL);
 }
 
 G_MODULE_EXPORT gchar* g_module_check_init(GModule *module);
