@@ -34,6 +34,30 @@ extern "C" {
 #define __attribute__(x)
 #endif
 
+/**
+ * CA_MAJOR:
+ *
+ * Evaluates to the major version number of libcanberra.
+ */
+#define CA_MAJOR (0)
+
+/**
+ * CA_MINOR:
+ *
+ * Evaluates to the minor version number of libcanberra.
+ */
+#define CA_MINOR (17)
+
+/**
+ * CA_CHECK_VERSION:
+ *
+ * Evaluates to TRUE when the library version is newer than the
+ * specified parameters.
+ */
+#define CA_CHECK_VERSION(major,minor)                           \
+    ((CA_MAJOR > (major)) ||                                    \
+     (CA_MAJOR == (major) && CA_MINOR >= (minor)))
+
 /**
  * CA_PROP_MEDIA_NAME:
  *
@@ -184,6 +208,72 @@ extern "C" {
  */
 #define CA_PROP_WINDOW_ICON_NAME                   "window.icon_name"
 
+/**
+ * CA_PROP_WINDOW_X:
+ *
+ * If this sound event was triggered by a window on the screen, the X
+ * position of the window measured from the top left corner of the
+ * screen to the top left corner of the window.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_X                           "window.x"
+
+/**
+ * CA_PROP_WINDOW_Y:
+ *
+ * If this sound event was triggered by a window on the screen, the y
+ * position of the window measured from the top left corner of the
+ * screen to the top left corner of the window.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_Y                           "window.y"
+
+/**
+ * CA_PROP_WINDOW_WIDTH:
+ *
+ * If this sound event was triggered by a window on the screen, the
+ * pixel width of the window.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_WIDTH                       "window.width"
+
+/**
+ * CA_PROP_WINDOW_HEIGHT:
+ *
+ * If this sound event was triggered by a window on the screen, the
+ * pixel height of the window.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_HEIGHT                      "window.height"
+
+/**
+ * CA_PROP_WINDOW_HPOS:
+ *
+ * If this sound event was triggered by a window on the screen, the X
+ * position of the center of the window as fractional value between 0
+ * and 1, formatted as string, 0 reflecting the left side of the
+ * screen, 1 the right side.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_HPOS                        "window.hpos"
+
+/**
+ * CA_PROP_WINDOW_VPOS:
+ *
+ * If this sound event was triggered by a window on the screen, the Y
+ * position of the center of the window as fractional value between 0
+ * and 1, formatted as string, 0 reflecting the top side of the
+ * screen, 1 the bottom side.
+ *
+ * Since: 0.17
+ */
+#define CA_PROP_WINDOW_VPOS                        "window.vpos"
+
 /**
  * CA_PROP_WINDOW_X11_DISPLAY:
  *
@@ -366,6 +456,23 @@ extern "C" {
  */
 #define CA_PROP_CANBERRA_ENABLE                    "canberra.enable"
 
+/**
+ * CA_PROP_CANBERRA_FORCE_CHANNEL:
+ *
+ * A special property that can be used to control on which channel a
+ * sound is played. The value should be one of mono, front-left,
+ * front-right, front-center, rear-left, rear-right, rear-center, lfe,
+ * front-left-of-center, front-right-of-center, side-left, side-right,
+ * top-center, top-front-left, top-front-right, top-front-center,
+ * top-rear-left, top-rear-right, top-rear-center. This property is
+ * only honoured by some backends, other backends may choose to ignore
+ * it completely.
+ *
+ * If the list of properties is handed on to the sound server this
+ * property is stripped from it.
+ */
+#define CA_PROP_CANBERRA_FORCE_CHANNEL             "canberra.force_channel"
+
 /**
  * ca_context:
  *
@@ -415,7 +522,8 @@ enum {
     CA_ERROR_INTERNAL = -15,
     CA_ERROR_DISABLED = -16,
     CA_ERROR_FORKED = -17,
-    _CA_ERROR_MAX = -18
+    CA_ERROR_DISCONNECTED = -18,
+    _CA_ERROR_MAX = -19
 };
 
 /**
@@ -443,6 +551,7 @@ int ca_context_play(ca_context *c, uint32_t id, ...) __attribute__((sentinel));
 int ca_context_cache_full(ca_context *c, ca_proplist *p);
 int ca_context_cache(ca_context *c, ...) __attribute__((sentinel));
 int ca_context_cancel(ca_context *c, uint32_t id);
+int ca_context_playing(ca_context *c, uint32_t id, int *playing);
 
 const char *ca_strerror(int code);
 
