@@ -804,6 +804,7 @@ int driver_play(ca_context *c, uint32_t id, ca_proplist *proplist, ca_finish_cal
     int ret;
     pa_operation *o;
     char *sp;
+    pa_buffer_attr ba;
 
     ca_return_val_if_fail(c, CA_ERROR_INVALID);
     ca_return_val_if_fail(proplist, CA_ERROR_INVALID);
@@ -999,7 +1000,15 @@ int driver_play(ca_context *c, uint32_t id, ca_proplist *proplist, ca_finish_cal
     if (volume_set)
         pa_cvolume_set(&cvol, ss.channels, v);
 
-    if (pa_stream_connect_playback(out->stream, NULL, NULL,
+    /* Make sure we get the longest latency possible, to minimize CPU
+     * consumption */
+    ba.maxlength = (uint32_t) -1;
+    ba.tlength = (uint32_t) -1;
+    ba.prebuf = (uint32_t) -1;
+    ba.minreq = (uint32_t) -1;
+    ba.fragsize = (uint32_t) -1;
+
+    if (pa_stream_connect_playback(out->stream, NULL, &ba,
 #ifdef PA_STREAM_FAIL_ON_SUSPEND
                                    PA_STREAM_FAIL_ON_SUSPEND
 #else
@@ -1271,3 +1280,33 @@ finish_unlocked:
 
     return ret;
 }
+
+int driver_playing(ca_context *c, uint32_t id, int *playing) {
+    struct private *p;
+    struct outstanding *out;
+
+    ca_return_val_if_fail(c, CA_ERROR_INVALID);
+    ca_return_val_if_fail(c->private, CA_ERROR_STATE);
+    ca_return_val_if_fail(playing, CA_ERROR_INVALID);
+
+    p = PRIVATE(c);
+
+    *playing = 0;
+
+    ca_mutex_lock(p->outstanding_mutex);
+
+    for (out = p->outstanding; out; out = out->next) {
+
+        if (out->type == OUTSTANDING_UPLOAD ||
+            out->id != id ||
+            out->sink_input == PA_INVALID_INDEX)
+            continue;
+
+        *playing = 1;
+        break;
+    }
+
+    ca_mutex_unlock(p->outstanding_mutex);
+
+    return CA_SUCCESS;
+}
